@@ -763,6 +763,10 @@ gnumeric_application_finalize (GObject *obj)
 	g_slist_foreach (application->history_list, (GFunc)g_free, NULL);
 	g_slist_free (application->history_list);
 	application->history_list = NULL;
+
+	g_free (application->clipboard_cut_range);
+	application->clipboard_cut_range = NULL;
+
 	app = NULL;
 	G_OBJECT_CLASS (parent_klass)->finalize (obj);
 }
