@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -243,7 +243,7 @@ paste_object (GnmPasteTarget const *pt, SheetObject const *src, int left, int to
  * paste a region.
  *
  * returns : TRUE if there was a problem.
- */
+ **/
 gboolean
 clipboard_paste_region (GnmCellRegion const *content,
 			GnmPasteTarget const *pt,
@@ -367,7 +367,7 @@ clipboard_paste_region (GnmCellRegion const *content,
 			GnmExprRewriteInfo   rwinfo;
 			GnmExprRelocateInfo *rinfo;
 
-			rwinfo.type = GNM_EXPR_REWRITE_RELOCATE;
+			rwinfo.rw_type = GNM_EXPR_REWRITE_EXPR;
 			rinfo = &rwinfo.u.relocate;
 			rinfo->origin_sheet = rinfo->target_sheet = pt->sheet;
 
