@@ -1,5 +1,5 @@
 /**
- * dialog-col-row.c:  Sets the magnification factor
+ * dialog-col-row.c:  group/ungroup dialog
  *
  * Author:
  *        Andreas J. Guelzow <aguelzow@taliesin.ca>
@@ -23,6 +23,7 @@
 #include <gnumeric-config.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <commands.h>
@@ -31,6 +32,8 @@
 #include <workbook-edit.h>
 
 #include <glade/glade.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtktogglebutton.h>
 
 #define COL_ROW_DIALOG_KEY "col-row-dialog"
 
@@ -99,7 +102,7 @@ dialog_col_row (WorkbookControlGUI *wbcg,  char const *operation,
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, COL_ROW_DIALOG_KEY))
 		return NULL;
-	gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"colrow.glade", NULL, NULL);
 	if (gui == NULL)
 		return NULL;
@@ -124,7 +127,7 @@ dialog_col_row (WorkbookControlGUI *wbcg,  char const *operation,
 
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
-		"row-height.html");
+		GNUMERIC_HELP_LINK_GROUP_UNGROUP);
 
 	g_signal_connect (G_OBJECT (state->dialog),
 		"destroy",
