@@ -47,7 +47,7 @@
 #include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeselection.h>
-#include <gtk/gtkclist.h>
+#include <gtk/gtklabel.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktable.h>
@@ -77,8 +77,10 @@ typedef struct {
 	GtkWidget           *delete_button;
 	GtkWidget           *model_button;
 	GtkWidget           *scenario_name_entry;
-	GnmExprEntry	    *lhs_entry;
-	GnmExprEntry	    *rhs_entry;
+	struct {
+		GnmExprEntry	*entry;
+		GtkWidget	*label;
+	} lhs, rhs;
 	GtkComboBox       *type_combo;
 	GtkComboBox       *algorithm_combo;
 	GtkTreeView            *constraint_list;
@@ -103,8 +105,8 @@ typedef struct {
 	SolverModelType     type;
 } algorithm_def_t;
 
-static algorithm_def_t algorithm_defs [] = {
-	{ N_("Simplex (LP Solve 5.0)"), LPSolve, SolverLPModel },
+static const algorithm_def_t algorithm_defs [] = {
+	{ N_("Simplex (LP Solve)"), LPSolve, SolverLPModel },
 	{ N_("Revised Simplex (GLPK 4.5)"), GLPKSimplex, SolverLPModel },
 	{ N_("< Not available >"), QPDummy, SolverQPModel },
 	{ NULL, 0, 0 }
@@ -116,14 +118,14 @@ typedef struct {
 	Sheet    *sheet;
 } constraint_conversion_t;
 
-static const char *problem_type_group[] = {
+static const char *const problem_type_group[] = {
 	"min_button",
 	"max_button",
 	"equal_to_button",
 	NULL
 };
 
-static const char *model_type_group[] = {
+static const char *const model_type_group[] = {
 	"lp_model_button",
 	"qp_model_button",
 	"nlp_model_button",
@@ -132,7 +134,6 @@ static const char *model_type_group[] = {
 
 static GList *lp_alg_name_list = NULL;
 static GList *qp_alg_name_list = NULL;
-const gchar *solver_max_time_err;
 
 /**
  * is_hom_row_or_col_ref:
@@ -176,12 +177,6 @@ is_hom_row_or_col_ref (GnmExprEntry *entry_1, GnmExprEntry *entry_2,
 	return res;
 }
 
-/**
- * dialog_set_sec_button_sensitivity:
- * @dummy:
- * @state:
- *
- **/
 static void
 dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 				   SolverState *state)
@@ -190,13 +185,13 @@ dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 	gboolean select_ready;
 
 	select_ready = (state->constr != NULL);
-	ready = gnm_expr_entry_is_cell_ref (state->lhs_entry, state->sheet,
+	ready = gnm_expr_entry_is_cell_ref (state->lhs.entry, state->sheet,
 					    TRUE) &&
 		((gtk_combo_box_get_active (state->type_combo)
 		  == SolverINT)
 		 || (gtk_combo_box_get_active (state->type_combo)
 		     == SolverBOOL)
-		 || (is_hom_row_or_col_ref (state->lhs_entry, state->rhs_entry,
+		 || (is_hom_row_or_col_ref (state->lhs.entry, state->rhs.entry,
 					    state->sheet)));
 
 	gtk_widget_set_sensitive (state->add_button, ready);
@@ -204,16 +199,6 @@ dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 	gtk_widget_set_sensitive (state->delete_button, select_ready);
 }
 
-/**
- * constraint_select_click:
- * @clist:
- * @row:
- * @column:
- * @event:
- * state:
- *
- **/
-
 static void
 constraint_select_click (GtkTreeSelection *Selection,
 			 SolverState    *state)
@@ -232,14 +217,14 @@ constraint_select_click (GtkTreeSelection *Selection,
 		return; /* Fail? */
 
 	range_init_value (&range, state->constr->lhs_value);
-	gnm_expr_entry_load_from_range (state->lhs_entry, state->sheet,&range);
+	gnm_expr_entry_load_from_range (state->lhs.entry, state->sheet,&range);
 
 	if (state->constr->type != SolverINT && state->constr->type != SolverBOOL) {
 		range_init_value (&range, state->constr->rhs_value);
-		gnm_expr_entry_load_from_range (state->rhs_entry,
+		gnm_expr_entry_load_from_range (state->rhs.entry,
 						state->sheet, &range);
 	} else
-		gnm_expr_entry_load_from_text (state->rhs_entry, "");
+		gnm_expr_entry_load_from_text (state->rhs.entry, "");
 
 	gtk_combo_box_set_active (state->type_combo, state->constr->type);
 }
@@ -292,13 +277,13 @@ constraint_fill_row (SolverState *state, GtkListStore *store, GtkTreeIter *iter)
 	constraint_t *the_constraint = g_new (constraint_t, 1);
 
 	the_constraint->lhs_value = gnm_expr_entry_parse_as_value
-		(state->lhs_entry, state->sheet);
+		(state->lhs.entry, state->sheet);
 	the_constraint->type = gtk_combo_box_get_active
 		(state->type_combo);
 	if ((the_constraint->type != SolverINT) &&
 	    (the_constraint->type != SolverBOOL)) {
 		the_constraint->rhs_value = gnm_expr_entry_parse_as_value
-			(state->rhs_entry, state->sheet);
+			(state->rhs.entry, state->sheet);
 
 /* FIXME: We are dropping cross sheet references!! */
 		text = write_constraint_str
@@ -330,32 +315,15 @@ constraint_fill_row (SolverState *state, GtkListStore *store, GtkTreeIter *iter)
 	return the_constraint;
 }
 
-/**
- * cb_dialog_add_clicked:
- * @button:
- * @state:
- *
- *
- **/
 static void
-cb_dialog_add_clicked (G_GNUC_UNUSED GtkWidget *button,
-		       SolverState *state)
+cb_dialog_add_clicked (SolverState *state)
 {
-	GtkListStore *store;
-	GtkTreeIter iter;
-
-	store = GTK_LIST_STORE (gtk_tree_view_get_model (state->constraint_list));
+	GtkTreeIter   iter;
+	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (state->constraint_list));
 	gtk_list_store_append (store, &iter);
 	constraint_fill_row (state, store, &iter);
 }
 
-/**
- * cb_dialog_change_clicked:
- * @button:
- * @state:
- *
- *
- **/
 static void
 cb_dialog_change_clicked (GtkWidget *button, SolverState *state)
 {
@@ -372,12 +340,6 @@ cb_dialog_change_clicked (GtkWidget *button, SolverState *state)
 	state->constr = constraint_fill_row (state, (GtkListStore *)store, &iter);
 }
 
-/**
- * dialog_set_main_button_sensitivity:
- * @dummy:
- * @state:
- *
- **/
 static void
 dialog_set_main_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 				    SolverState *state)
@@ -391,38 +353,16 @@ dialog_set_main_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 	gtk_widget_set_sensitive (state->solve_button, TRUE);
 }
 
-/**
- * cb_dialog_set_rhs_sensitivity:
- * @dummy:
- * @state:
- *
- **/
 static void
 cb_dialog_set_rhs_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 			       SolverState *state)
 {
-	return;
-
-/* FIXME: We would like to disable the rhs when appropriate. */
-/* Unfortunately this confuses the widget:                   */
-
-	if ((gtk_combo_box_get_active (state->type_combo)
-		  == SolverINT)
-		 || (gtk_combo_box_get_active (state->type_combo)
-		     == SolverBOOL)) {
-		gtk_widget_set_sensitive (GTK_WIDGET (state->rhs_entry), FALSE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (state->rhs_entry), TRUE);
-	}
+	int t = gtk_combo_box_get_active (state->type_combo);
+	gboolean sensitive = (t != SolverINT && t != SolverBOOL);
+	gtk_widget_set_sensitive (GTK_WIDGET (state->rhs.entry), sensitive);
+	gtk_widget_set_sensitive (GTK_WIDGET (state->rhs.label), sensitive);
 }
 
-/**
- * cb_dialog_lp_clicked:
- * @button:
- * @state:
- *
- *
- **/
 static void
 cb_dialog_model_type_clicked (G_GNUC_UNUSED GtkWidget *button,
 			      SolverState *state)
@@ -445,7 +385,7 @@ cb_dialog_model_type_clicked (G_GNUC_UNUSED GtkWidget *button,
 		l = qp_alg_name_list;
 		gtk_widget_set_sensitive (GTK_WIDGET (state->solve_button),
 					  FALSE);
-		gnumeric_notice_nonmodal ((GtkWindow *) state->dialog,
+		go_gtk_notice_nonmodal_dialog ((GtkWindow *) state->dialog,
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_INFO,
 					  _("Looking for a subject for your "
@@ -466,14 +406,6 @@ cb_dialog_model_type_clicked (G_GNUC_UNUSED GtkWidget *button,
 	gtk_combo_box_set_active (state->algorithm_combo ,0);
 }
 
-/**
- * free_original_values:
- * @ov:
- * @user_data:  (will be NULL)
- *
- * Destroy original savec values.
- *
- **/
 static void
 free_original_values (GSList *ov, G_GNUC_UNUSED gpointer user_data)
 {
@@ -481,15 +413,6 @@ free_original_values (GSList *ov, G_GNUC_UNUSED gpointer user_data)
 	g_slist_free (ov);
 }
 
-/**
- * dialog_destroy:
- * @window:
- * @focus_widget:
- * @state:
- *
- * Destroy the dialog and associated data structures.
- *
- **/
 static gboolean
 dialog_destroy (GtkObject *w, SolverState  *state)
 {
@@ -542,12 +465,6 @@ dialog_destroy (GtkObject *w, SolverState  *state)
 	return FALSE;
 }
 
-/**
- * restore_original_values:
- * @input_cells:
- * @ov:
- *
- **/
 static void
 restore_original_values (GSList *input_cells, GSList *ov)
 {
@@ -561,14 +478,6 @@ restore_original_values (GSList *input_cells, GSList *ov)
 	}
 }
 
-
-/**
- * cb_dialog_cancel_clicked:
- * @button:
- * @state:
- *
- * Close (destroy) the dialog
- **/
 static void
 cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button, SolverState *state)
 {
@@ -594,13 +503,6 @@ cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button, SolverState *state)
 	gtk_widget_destroy (state->dialog);
 }
 
-/**
- * cb_dialog_close_clicked:
- * @button:
- * @state:
- *
- * Close (destroy) the dialog
- **/
 static void
 cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 			 SolverState *state)
@@ -609,11 +511,6 @@ cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 	gtk_widget_destroy (state->dialog);
 }
 
-/*
- *  grab_cells:
- *  @cell:
- *  @data: pointer to a data_set_t
- */
 static GnmValue *
 grab_cells (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
 {
@@ -625,20 +522,15 @@ grab_cells (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
 	return NULL;
 }
 
-/*
- *  check_int_constraints:
- *  @cell:
- *  @data: pointer to a data_set_t
- */
 static gchar const *
-check_int_constraints (GnmValue *input_range, GtkTreeView *constraint_list)
+check_int_constraints (GnmValue *input_range, SolverState *state)
 {
 	GtkTreeModel *store;
 	GtkTreeIter iter;
 	const constraint_t *a_constraint;
 	const gchar *text;
 
-	store = gtk_tree_view_get_model (constraint_list);
+	store = gtk_tree_view_get_model (state->constraint_list);
 	if (gtk_tree_model_get_iter_first (store, &iter))
 		do {
 			gtk_tree_model_get (store, &iter, 0, &text, 1, &a_constraint, -1);
@@ -647,8 +539,7 @@ check_int_constraints (GnmValue *input_range, GtkTreeView *constraint_list)
 			if ((a_constraint->type != SolverINT) &&
 				(a_constraint->type != SolverBOOL))
 				continue;
-			if (!global_range_contained (a_constraint->lhs_value,
-							 input_range))
+			if (!global_range_contained (state->sheet, a_constraint->lhs_value, input_range))
 				return text;
 		} while (gtk_tree_model_iter_next (store, &iter));
 	return NULL;
@@ -788,15 +679,18 @@ cb_destroy (gpointer data, G_GNUC_UNUSED gpointer user_data)
 }
 
 
-static void
+/* Returns FALSE if the reports deleted the current sheet
+ * and forced the dialog to die */
+static gboolean
 solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 {
 	SolverOptions *opt = &res->param->options;
 	gchar         *err = NULL;
 
+	g_object_add_weak_pointer (G_OBJECT (state->dialog), (gpointer)&state);
 	switch (res->status) {
 	case SolverOptimal :
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_INFO,
@@ -805,7 +699,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 			   "satisfied.\n"));
 		if ((opt->sensitivity_report || opt->limits_report)
 		    && res->ilp_flag)
-			gnumeric_notice_nonmodal
+			go_gtk_notice_nonmodal_dialog
 				((GtkWindow *) state->dialog,
 				 &(state->warning_dialog),
 				 GTK_MESSAGE_INFO,
@@ -823,7 +717,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				      opt->dual_program_report);
 		break;
 	case SolverUnbounded :
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_WARNING,
@@ -837,7 +731,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				      opt->dual_program_report);
 		break;
 	case SolverInfeasible :
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_WARNING,
@@ -852,7 +746,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				      opt->dual_program_report);
 		break;
 	case SolverMaxIterExc :
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR,
@@ -866,7 +760,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				      opt->dual_program_report);
 		break;
 	case SolverMaxTimeExc :
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR,
@@ -879,16 +773,20 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				      opt->dual_program_report);
 		break;
 	default:
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR, errmsg);
 		break;
 	}
+	if (NULL != state)
+		g_object_remove_weak_pointer (G_OBJECT (state->dialog), (gpointer)&state);
+
 	if (err)
-		gnumeric_notice_nonmodal
-			((GtkWindow *) state->dialog,
+		go_gtk_notice_nonmodal_dialog (state ? ((GtkWindow *) state->dialog) : NULL,
 			 &(state->warning_dialog), GTK_MESSAGE_ERROR, err);
+
+	return state != NULL;
 }
 
 static void
@@ -946,7 +844,7 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 						     state->sheet);
 
 	if (target_range == NULL || input_range == NULL) {
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			  &(state->warning_dialog),
 			  GTK_MESSAGE_ERROR, _("You have not specified "
@@ -965,7 +863,7 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 
 	/* Check that the target cell type is number. */
 	if (! cell_is_number (param->target_cell)) {
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR, _("Target cell should contain "
@@ -1041,15 +939,14 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	dual_program = FALSE;
 	param->options.dual_program_report = dual_program;
 
-	name = check_int_constraints (input_range, state->constraint_list);
-
+	name = check_int_constraints (input_range, state);
 	if (name != NULL) {
 		char *str;
 
 		str = g_strdup_printf
 			(_("Constraint `%s' is for a cell that "
 			   "is not an input cell."), name);
-		gnumeric_notice_nonmodal ((GtkWindow *) state->dialog,
+		go_gtk_notice_nonmodal_dialog ((GtkWindow *) state->dialog,
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR, str);
 		g_free (str);
@@ -1066,7 +963,7 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	}
 	param->constraints = conv.c_list;
 	if (param->constraints == NULL) {
-		gnumeric_notice_nonmodal
+		go_gtk_notice_nonmodal_dialog
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR, _("No constraints defined for "
@@ -1085,15 +982,18 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 
 	if (res != NULL) {
 		state->cancelled = FALSE;
-		solver_reporting (state, res, errmsg);
-		if (res->status == SolverOptimal && 
+
+		/* WARNING : The dialog may be deleted by the reports
+		 * solver_reporting will return FALSE if state is gone and cleared */
+		if (solver_reporting (state, res, errmsg) &&
+		    res->status == SolverOptimal && 
 		    param->options.add_scenario)
 			solver_add_scenario (state, res,
 					     param->options.scenario_name);
 
 		solver_results_free (res);
 	} else
-		gnumeric_notice_nonmodal (GTK_WINDOW (state->dialog),
+		go_gtk_notice_nonmodal_dialog (GTK_WINDOW (state->dialog),
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_ERROR, errmsg);
 out:
@@ -1126,10 +1026,6 @@ dialog_init (SolverState *state)
 
 	param = state->sheet->solver_parameters;
 
-	solver_max_time_err = _("The maximum time exceeded. The optimal "
-				"value could not be found in given "
-				"time.");
-
 	if (lp_alg_name_list == NULL) {
 		for (i = 0; algorithm_defs [i].name; i++)
 			switch (algorithm_defs [i].type) {
@@ -1148,7 +1044,7 @@ dialog_init (SolverState *state)
 			}
 	}
 
-	state->gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (state->wbcg),
+	state->gui = gnm_glade_xml_new (GO_CMD_CONTEXT (state->wbcg),
 		"solver.glade", NULL, NULL);
         if (state->gui == NULL)
                 return TRUE;
@@ -1157,7 +1053,6 @@ dialog_init (SolverState *state)
         if (state->dialog == NULL)
                 return TRUE;
 
-
 /*  buttons  */
 	state->solve_button  = glade_xml_get_widget (state->gui, "solvebutton");
 	g_signal_connect (G_OBJECT (state->solve_button), "clicked",
@@ -1177,8 +1072,9 @@ dialog_init (SolverState *state)
 				   GNUMERIC_HELP_LINK_SOLVER);
 
 	state->add_button  = glade_xml_get_widget (state->gui, "addbutton");
-	g_signal_connect (G_OBJECT (state->add_button), "clicked",
-			  G_CALLBACK (cb_dialog_add_clicked), state);
+	gtk_button_set_alignment (GTK_BUTTON (state->add_button), 0.5, .5);
+	g_signal_connect_swapped (G_OBJECT (state->add_button), "clicked",
+		G_CALLBACK (cb_dialog_add_clicked), state);
 
 	state->change_button = glade_xml_get_widget (state->gui,
 						     "changebutton");
@@ -1187,6 +1083,7 @@ dialog_init (SolverState *state)
 
 	state->delete_button = glade_xml_get_widget (state->gui,
 						     "deletebutton");
+	gtk_button_set_alignment (GTK_BUTTON (state->delete_button), 0.5, .5);
 	g_signal_connect (G_OBJECT (state->delete_button), "clicked",
 			  G_CALLBACK (cb_dialog_delete_clicked), state);
 
@@ -1272,36 +1169,44 @@ dialog_init (SolverState *state)
 
 /* lhs_entry */
 	table = GTK_TABLE (glade_xml_get_widget (state->gui, "edit-table"));
-	state->lhs_entry = gnm_expr_entry_new (state->wbcg, TRUE);
-	gnm_expr_entry_set_flags (state->lhs_entry,
+	state->lhs.entry = gnm_expr_entry_new (state->wbcg, TRUE);
+	gnm_expr_entry_set_flags (state->lhs.entry,
 		GNM_EE_SINGLE_RANGE |
 		GNM_EE_SHEET_OPTIONAL, GNM_EE_MASK);
-	gtk_table_attach (table, GTK_WIDGET (state->lhs_entry),
+	gtk_table_attach (table, GTK_WIDGET (state->lhs.entry),
 			  0, 1, 1, 2,
 			  GTK_EXPAND | GTK_FILL, 0,
 			  0, 0);
- 	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
-				  GTK_WIDGET (state->lhs_entry));
-	gtk_widget_show (GTK_WIDGET (state->lhs_entry));
-	g_signal_connect_after (G_OBJECT (state->lhs_entry),
+	state->lhs.label = glade_xml_get_widget (state->gui, "lhs_label");
+	gtk_label_set_mnemonic_widget (GTK_LABEL (state->lhs.label),
+		GTK_WIDGET (state->lhs.entry));
+	gtk_widget_show (GTK_WIDGET (state->lhs.entry));
+	g_signal_connect_after (G_OBJECT (state->lhs.entry),
 		"changed",
 		G_CALLBACK (dialog_set_sec_button_sensitivity), state);
+	g_signal_connect_swapped (
+		gnm_expr_entry_get_entry (GNM_EXPR_ENTRY (state->lhs.entry)),
+		"activate", G_CALLBACK (cb_dialog_add_clicked), state);
 
 /* rhs_entry */
 	table = GTK_TABLE (glade_xml_get_widget (state->gui, "edit-table"));
-	state->rhs_entry = gnm_expr_entry_new (state->wbcg, TRUE);
-	gnm_expr_entry_set_flags (state->rhs_entry,
+	state->rhs.entry = gnm_expr_entry_new (state->wbcg, TRUE);
+	gnm_expr_entry_set_flags (state->rhs.entry,
 		GNM_EE_SINGLE_RANGE |
 		GNM_EE_SHEET_OPTIONAL, GNM_EE_MASK);
-	gtk_table_attach (table, GTK_WIDGET (state->rhs_entry),
+	gtk_table_attach (table, GTK_WIDGET (state->rhs.entry),
 			  2, 3, 1, 2,
 			  GTK_EXPAND | GTK_FILL, 0, 0, 0);
- 	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
-				  GTK_WIDGET (state->rhs_entry));
-	gtk_widget_show (GTK_WIDGET (state->rhs_entry));
-	g_signal_connect_after (G_OBJECT (state->rhs_entry),
+	gtk_widget_show (GTK_WIDGET (state->rhs.entry));
+	state->rhs.label = glade_xml_get_widget (state->gui, "rhs_label");
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (state->rhs.label), GTK_WIDGET (state->rhs.entry));
+	g_signal_connect_after (G_OBJECT (state->rhs.entry),
 		"changed",
 		G_CALLBACK (dialog_set_sec_button_sensitivity), state);
+	g_signal_connect_swapped (
+		gnm_expr_entry_get_entry (GNM_EXPR_ENTRY (state->rhs.entry)),
+		"activate", G_CALLBACK (cb_dialog_add_clicked), state);
 
 /* type_menu */
 	state->type_combo = GTK_COMBO_BOX
@@ -1325,8 +1230,8 @@ dialog_init (SolverState *state)
 	gtk_tree_view_set_model (state->constraint_list, GTK_TREE_MODEL(store));
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (
-								_("Subject to the Constraints:"),
-								renderer, "text", 0, NULL);
+			_("Subject to the Constraints:"),
+			renderer, "text", 0, NULL);
 	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_append_column (state->constraint_list, column);
 
@@ -1440,7 +1345,7 @@ dialog_solver (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->cancelled      = TRUE;
 
 	if (dialog_init (state)) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the Solver dialog."));
 		g_free (state);
 		return;
