@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -62,11 +62,16 @@ so_polygon_view_set_bounds (SheetObjectView *sov, double const *coords, gboolean
 	if (visible) {
 		SheetObject		*so   = sheet_object_view_get_so (sov);
 		GnmSOPolygon const	*sop  = GNM_SO_POLYGON (so);
-		unsigned		 i    = sop->points->len / 2;
-		FooCanvasPoints		*pts  = foo_canvas_points_new (i);
+		unsigned		 i;
+		FooCanvasPoints		*pts;
 		double *dst, x_scale, y_scale, x_translate, y_translate;
 		double const *src;
 
+		if (sop->points == NULL)
+			return;
+
+		i = sop->points->len / 2;
+		pts = foo_canvas_points_new (i);
 		x_scale = fabs (coords[2] - coords[0]);
 		y_scale = fabs (coords[3] - coords[1]);
 		x_translate = MIN (coords[0], coords[2]),
@@ -205,14 +210,6 @@ gnm_so_polygon_read_xml_dom (SheetObject *so, char const *typename,
 		read_xml_dom (so, typename, ctxt, node);
 }
 
-static gboolean
-gnm_so_polygon_write_xml_dom (SheetObject const *so,
-			      XmlParseContext const *ctxt, xmlNodePtr node)
-{
-	/* TODO */
-	return gnm_so_polygon_parent_class->write_xml_dom (so, ctxt, node);
-}
-
 static void
 gnm_so_polygon_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
@@ -306,7 +303,6 @@ gnm_so_polygon_class_init (GObjectClass *gobject_class)
 	gobject_class->set_property	= gnm_so_polygon_set_property;
 	gobject_class->get_property	= gnm_so_polygon_get_property;
 	so_class->read_xml_dom		= gnm_so_polygon_read_xml_dom;
-	so_class->write_xml_dom		= gnm_so_polygon_write_xml_dom;
 	so_class->write_xml_sax		= gnm_so_polygon_write_xml_sax;
 	so_class->copy			= gnm_so_polygon_copy;
 	so_class->rubber_band_directly	= FALSE;
