@@ -867,44 +867,6 @@ gnm_widget_set_cursor_type (GtkWidget *w, GdkCursorType ct)
 
 /* ------------------------------------------------------------------------- */
 
-/**
- * gnumeric_dialog_add_button
- *
- * Code from gedit
- *
- * Creates and adds a button with stock image to the action area of an existing dialog.
- * 
- * dialog : dialog you want to add a button
- * text : button label
- * stock_id : stock icon id
- * response_id : respond id when button clicked
- *
- * return : newly created button
- *
- **/
-
-GtkWidget*
-gnumeric_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stock_id,
-			    gint response_id)
-{
-	GtkWidget *button;
-
-	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
-	g_return_val_if_fail (text != NULL, NULL);
-	g_return_val_if_fail (stock_id != NULL, NULL);
-
-	button = go_gtk_button_new_with_stock (text, stock_id);
-	g_return_val_if_fail (button != NULL, NULL);
-
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-
-	gtk_widget_show (button);
-
-	gtk_dialog_add_action_widget (dialog, button, response_id);	
-
-	return button;
-}
-
 /**
  * gnumeric_message_dialog_new :
  *
