@@ -14,6 +14,10 @@
 #include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "libgnumeric.h"
+#ifdef WIN32
+#define POPT_STATIC
+#endif
+#include <popt.h>
 
 #include "command-context.h"
 #include <goffice/app/io-context.h>
@@ -289,6 +293,39 @@ cb_kill_wbcg (WorkbookControlGUI *wbcg)
 	return FALSE;
 }
 
+static void
+check_pango_attr_list_splice_bug (void)
+{
+	PangoAttrList *l1 = pango_attr_list_new ();
+	PangoAttrList *l2 = pango_attr_list_new ();
+	PangoAttribute *a = pango_attr_weight_new (1000);
+	PangoAttrIterator *it;
+	gboolean buggy;
+
+	a->start_index = 4;
+	a->end_index = 5;
+	pango_attr_list_insert (l1, a);
+
+	pango_attr_list_splice (l1, l2, 0, 1);
+	pango_attr_list_unref (l2);
+
+	it = pango_attr_list_get_iterator (l1);
+	if (pango_attr_iterator_next (it)) {
+		gint s, e;
+		pango_attr_iterator_range (it, &s, &e);
+		buggy = (s != 5 || e != 6);
+	} else
+		buggy = TRUE;
+	pango_attr_iterator_destroy (it);
+	pango_attr_list_unref (l1);
+
+	if (buggy)
+		g_warning (_("The Pango library present on your system is buggy, see\n"
+			     "http://bugzilla.gnome.org/show_bug.cgi?id=316054\n"
+			     "Editing rich text therefore does not work well.  Please check\n"
+			     "with your distribution if a fixed Pango library is available."));
+}
+
 int
 main (int argc, char const *argv [])
 {
@@ -299,14 +336,12 @@ main (int argc, char const *argv [])
 	WorkbookView *wbv;
 	GSList *wbcgs_to_kill = NULL;
 	poptContext ctx;
-	gchar **args;
-	
-	args = go_shell_argv_to_glib_encoding (argc, argv);
+	gchar const **args = go_shell_argv_to_glib_encoding (argc, argv);
 
 	gnm_pre_parse_init (args[0]);
 
-	ctx = gnumeric_arg_parse (argc, (gchar const **) args);
-        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	ctx = gnumeric_arg_parse (argc, args);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	if (gnumeric_show_version) {
 		g_print (_("gnumeric version '%s'\ndatadir := '%s'\nlibdir := '%s'\n"),
@@ -316,8 +351,10 @@ main (int argc, char const *argv [])
 
 	with_gui = !func_def_file && !func_state_file && !split_funcdocs;
 
-	if (with_gui)
+	if (with_gui) {
+		check_pango_attr_list_splice_bug ();
 		gnm_session_init (args[0]);
+	}
 
 	/* TODO: Use the ioc.  Do this before calling handle_paint_events */
 	gnm_common_init (TRUE);
@@ -439,14 +476,7 @@ main (int argc, char const *argv [])
 #ifdef WITH_GNOME
 	bonobo_ui_debug_shutdown ();
 #endif
-
-	if (argv != (gchar const **) args) {
-		gint i;
-
-		for (i = 0; i < argc; ++i)
-			g_free (args[i]);
-		g_free (args);
-	}
+	go_shell_argv_to_glib_encoding_free ();
 
 	return 0;
 }
