@@ -39,8 +39,9 @@
 #include "number-match.h"
 #include "gutils.h"
 #include "parse-util.h"
-#include "format.h"
-#include "datetime.h"
+#include "gnm-format.h"
+#include <goffice/utils/datetime.h>
+#include <goffice/utils/go-glib-extras.h>
 
 #include <stdlib.h>
 #include <locale.h>
@@ -48,11 +49,11 @@
 #define SETUP_LOCALE_SWITCH char *oldlocale = NULL
 
 #define START_LOCALE_SWITCH if (parseoptions->locale) {\
-oldlocale = g_strdup(gnm_setlocale (LC_ALL, NULL)); \
-gnm_setlocale(LC_ALL, parseoptions->locale);}
+oldlocale = g_strdup(go_setlocale (LC_ALL, NULL)); \
+go_setlocale(LC_ALL, parseoptions->locale);}
 
 #define END_LOCALE_SWITCH if (oldlocale) {\
-gnm_setlocale(LC_ALL, oldlocale);\
+go_setlocale(LC_ALL, oldlocale);\
 g_free (oldlocale);}
 
 #define WARN_TOO_MANY_ROWS _("Too many rows in data to parse: %d")
@@ -206,10 +207,8 @@ stf_parse_options_free (StfParseOptions_t *parseoptions)
 		unsigned int ui;
 		GPtrArray *formats = parseoptions->formats;
 
-		for (ui = 0; ui < formats->len; ui++) {
-			GnmFormat *sf = g_ptr_array_index (formats, ui);
-			style_format_unref (sf);
-		}
+		for (ui = 0; ui < formats->len; ui++)
+			go_format_unref (g_ptr_array_index (formats, ui));
 		g_ptr_array_free (formats, TRUE);
 		parseoptions->formats = NULL;
 	}
@@ -237,7 +236,7 @@ static void
 compile_terminators (StfParseOptions_t *parseoptions)
 {
 	GSList *l;
-	GNM_SLIST_SORT (parseoptions->terminator, (GCompareFunc)long_string_first);
+	GO_SLIST_SORT (parseoptions->terminator, (GCompareFunc)long_string_first);
 
 	parseoptions->compiled_terminator.min = 255;
 	parseoptions->compiled_terminator.max = 0;
@@ -263,7 +262,7 @@ stf_parse_options_add_line_terminator (StfParseOptions_t *parseoptions, char con
 	g_return_if_fail (parseoptions != NULL);
 	g_return_if_fail (terminator != NULL && *terminator != 0);
 
-	GNM_SLIST_PREPEND (parseoptions->terminator, g_strdup (terminator));
+	GO_SLIST_PREPEND (parseoptions->terminator, g_strdup (terminator));
 	compile_terminators (parseoptions);
 }
 
@@ -281,11 +280,11 @@ stf_parse_options_remove_line_terminator (StfParseOptions_t *parseoptions, char
 
 	g_return_if_fail (parseoptions != NULL);
 
-	in_list = g_slist_find_custom (parseoptions->terminator, terminator, gnm_str_compare);
+	in_list = g_slist_find_custom (parseoptions->terminator, terminator, go_str_compare);
 
 	if (in_list) {
 		char *s = in_list->data;
-		GNM_SLIST_REMOVE (parseoptions->terminator, in_list->data);
+		GO_SLIST_REMOVE (parseoptions->terminator, in_list->data);
 		g_free (s);
 		compile_terminators (parseoptions);
 	}
@@ -303,7 +302,7 @@ stf_parse_options_clear_line_terminator (StfParseOptions_t *parseoptions)
 {
 	g_return_if_fail (parseoptions != NULL);
 
-	gnm_slist_free_custom (parseoptions->terminator, g_free);
+	go_slist_free_custom (parseoptions->terminator, g_free);
 	parseoptions->terminator = NULL;
 	compile_terminators (parseoptions);
 }
@@ -336,8 +335,8 @@ stf_parse_options_csv_set_separators (StfParseOptions_t *parseoptions, char cons
 	g_free (parseoptions->sep.chr);
 	parseoptions->sep.chr = g_strdup (character);
 
-	gnm_slist_free_custom (parseoptions->sep.str, g_free);
-	parseoptions->sep.str = gnm_slist_map (string, (GnmMapFunc)g_strdup);
+	go_slist_free_custom (parseoptions->sep.str, g_free);
+	parseoptions->sep.str = go_slist_map (string, (GOMapFunc)g_strdup);
 }
 
 void
@@ -1146,7 +1145,7 @@ stf_parse_sheet (StfParseOptions_t *parseoptions,
 {
 	int row, col;
 	unsigned int lrow, lcol;
-	GnmDateConventions const *date_conv;
+	GODateConventions const *date_conv;
 	GStringChunk *lines_chunk;
 	GPtrArray *lines, *line;
 
@@ -1165,6 +1164,8 @@ stf_parse_sheet (StfParseOptions_t *parseoptions,
 	lines_chunk = g_string_chunk_new (100 * 1024);
 	lines = stf_parse_general (parseoptions, lines_chunk, data, data_end,
 				   SHEET_MAX_ROWS);
+	if (lines == NULL)
+		return FALSE;
 	for (row = start_row, lrow = 0; lrow < lines->len ; row++, lrow++) {
 		col = start_col;
 		line = g_ptr_array_index (lines, lrow);
@@ -1198,10 +1199,14 @@ stf_parse_sheet (StfParseOptions_t *parseoptions,
 }
 
 GnmCellRegion *
-stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const *data_end)
+stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const *data_end,
+		  Workbook const *wb)
 {
+	static GODateConventions const default_conv = {FALSE};
+	GODateConventions const *date_conv = wb ? workbook_date_conv (wb) : &default_conv;
+
 	GnmCellRegion *cr;
-	CellCopyList *content = NULL;
+	GSList	 *content = NULL;
 	unsigned int row, colhigh = 0;
 	GStringChunk *lines_chunk;
 	GPtrArray *lines;
@@ -1220,9 +1225,6 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	for (row = 0; row < lines->len; row++) {
 		GPtrArray *line = g_ptr_array_index (lines, row);
 		unsigned int col, targetcol = 0;
-#warning "FIXME: We should not just assume the 1900 convention "
-		GnmDateConventions date_conv = {FALSE};
-
 		for (col = 0; col < line->len; col++) {
 			if (parseoptions->col_import_array == NULL ||
 			    parseoptions->col_import_array_len <= col ||
@@ -1230,23 +1232,19 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 				char *text = g_ptr_array_index (line, col);
 
 				if (text) {
-					CellCopy *ccopy;
+					GnmCellCopy *ccopy;
 					GnmValue *v;
-					GnmFormat *fmt = g_ptr_array_index
+					GOFormat *fmt = g_ptr_array_index
 						(parseoptions->formats, col);
 
-					v = format_match (text, fmt, &date_conv);
+					v = format_match (text, fmt, date_conv);
 					if (v == NULL) {
 						v = value_new_string (text);
 					}
 
-					ccopy = g_new (CellCopy, 1);
-					ccopy->type = CELL_COPY_TYPE_CELL;
-					ccopy->col_offset = targetcol;
-					ccopy->row_offset = row;
-					ccopy->u.cell = cell_new ();
-					cell_set_value(ccopy->u.cell, v);
-
+					ccopy = gnm_cell_copy_new (targetcol, row);
+					ccopy->val  = v;
+					ccopy->expr = NULL;
 					content = g_slist_prepend (content, ccopy);
 					targetcol++;
 					if (targetcol > colhigh)
@@ -1344,31 +1342,39 @@ static int
 count_character (GPtrArray *lines, gunichar c, double quantile)
 {
 	int *counts, res;
-	unsigned int ui;
+	unsigned int lno, cno;
 
 	if (lines->len == 0)
 		return 0;
 
 	counts = g_new (int, lines->len);
-	for (ui = 0; ui < lines->len; ui++) {
+	for (lno = cno = 0; lno < lines->len; lno++) {
 		int count = 0;
-		GPtrArray *boxline = g_ptr_array_index (lines, ui);
+		GPtrArray *boxline = g_ptr_array_index (lines, lno);
 		const char *line = g_ptr_array_index (boxline, 0);
 
+		/* Ignore empty lines.  */
+		if (*line == 0)
+			continue;
+
 		while (*line) {
 			if (g_utf8_get_char (line) == c)
 				count++;
 			line = g_utf8_next_char (line);
 		}
 
-		counts[ui] = count;
+		counts[cno++] = count;
 	}
 
-	qsort (counts, lines->len, sizeof (counts[0]), int_sort);
-	ui = (unsigned int)ceil (quantile * lines->len);
-	if (ui == lines->len)
-		ui--;
-	res = counts[ui];
+	if (cno == 0)
+		res = 0;
+	else {
+		unsigned int qi = (unsigned int)ceil (quantile * cno);
+		qsort (counts, cno, sizeof (counts[0]), int_sort);
+		if (qi == cno)
+			qi--;
+		res = counts[qi];
+	}
 
 	g_free (counts);
 
