@@ -18,6 +18,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
+ *
+ * Port to Maemo:
+ * 	Eduardo Lima  (eduardo.lima@indt.org.br)
+ * 	Renato Araujo (renato.filho@indt.org.br)
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -77,14 +81,51 @@
 #include <gsf/gsf-input.h>
 #include <string.h>
 
+#ifdef USE_HILDON
+#include <wbc-gtk.h>
+#include <hildon-lgpl/hildon-widgets/hildon-appview.h>
+#include <hildon-widgets/hildon-color-selector.h>
+#include <hildon-widgets/hildon-font-selection-dialog.h>
+#endif
+
 static GNM_ACTION_DEF (cb_file_new)
 {
 	GdkScreen *screen = gtk_window_get_screen (wbcg_toplevel (wbcg));
 	Workbook *wb = workbook_new_with_sheets
 		(gnm_app_prefs->initial_sheet_number);
+
+#ifdef USE_HILDON
+	WorkbookControl * wbc = (WorkbookControl *) wbcg;
+	Workbook *tmp_wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+
+	if (workbook_is_dirty (tmp_wb)) {
+		switch (wbcg_show_save_dialog (wbcg, tmp_wb, FALSE)) {
+
+			case GTK_RESPONSE_YES:
+			case GNM_RESPONSE_SAVE_ALL:
+				gui_file_save (wbcg, wb_control_view (WORKBOOK_CONTROL (wbcg)));
+				break;
+
+			case GTK_RESPONSE_NO:
+			case GNM_RESPONSE_DISCARD_ALL:
+				/* Do nothing */
+				break;
+			
+			default:  /* CANCEL */
+				return;
+		}
+	}
+
+	g_object_ref (wbcg);
+	g_object_unref (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
+	wb_control_set_view (wbc, NULL, wb);
+	wb_control_init_state (wbc);
+	sheet_update (wb_view_cur_sheet (wb_control_view (wbc)));
+#else
 	WorkbookControl *new_wbc = workbook_control_gui_new (NULL, wb, screen);
 	WorkbookControlGUI *new_wbcg = WORKBOOK_CONTROL_GUI (new_wbc);
 	wbcg_copy_toolbar_visibility (new_wbcg, wbcg);	
+#endif
 }
 
 static GNM_ACTION_DEF (cb_file_open)	{ gui_file_open (wbcg, NULL); }
@@ -316,6 +357,11 @@ static GNM_ACTION_DEF (cb_edit_duplicate_sheet)
 	cmd_clone_sheet (wbc, old_sheet);
 }
 
+#ifdef USE_HILDON
+static GNM_ACTION_DEF (cb_edit_undo) { command_undo (WORKBOOK_CONTROL (wbcg)); }
+static GNM_ACTION_DEF (cb_edit_redo) { command_redo (WORKBOOK_CONTROL (wbcg)); }
+#endif
+
 static void
 common_cell_goto (WorkbookControlGUI *wbcg, Sheet *sheet, GnmCellPos const *pos)
 {
@@ -522,10 +568,20 @@ static GNM_ACTION_DEF (cb_view_zoom_in)
 
 static GNM_ACTION_DEF (cb_view_fullscreen)
 {
+#ifdef USE_HILDON
+	if (wbcg->is_fullscreen || hildon_appview_get_fullscreen (HILDON_APPVIEW (wbcg->toplevel))) {
+		wbcg->is_fullscreen = FALSE;
+		hildon_appview_set_fullscreen (HILDON_APPVIEW (wbcg->toplevel), FALSE);
+	} else {
+		wbcg->is_fullscreen = TRUE;
+		hildon_appview_set_fullscreen (HILDON_APPVIEW (wbcg->toplevel), TRUE);
+	}
+#else
 	if (wbcg->is_fullscreen)
 		gtk_window_unfullscreen (wbcg_toplevel (wbcg));
 	else
 		gtk_window_fullscreen (wbcg_toplevel (wbcg));
+#endif
 }
 
 static GNM_ACTION_DEF (cb_view_zoom)	{ dialog_zoom (wbcg, wbcg_cur_sheet (wbcg)); }
@@ -1189,6 +1245,9 @@ toggle_font_attr (WorkbookControlGUI *wbcg, GtkToggleAction *act,
 		case MSTYLE_FONT_STRIKETHROUGH:
 			attr = pango_attr_strikethrough_new (val);
 			break;
+		case MSTYLE_FONT_SCRIPT:
+			attr = pango_attr_rise_new (5000 * val);
+			break;
 		}
 		wbcg_edit_add_markup (wbcg, attr);
 		return;
@@ -1201,6 +1260,7 @@ toggle_font_attr (WorkbookControlGUI *wbcg, GtkToggleAction *act,
 	case MSTYLE_FONT_ITALIC:	gnm_style_set_font_italic (new_style, val); break;
 	case MSTYLE_FONT_UNDERLINE:	gnm_style_set_font_uline (new_style, val); break;
 	case MSTYLE_FONT_STRIKETHROUGH: gnm_style_set_font_strike (new_style, val); break;
+	case MSTYLE_FONT_SCRIPT:	gnm_style_set_font_script (new_style, val); break;
 	};
 
 	cmd_selection_format (wbc, new_style, NULL, _("Set Font Style"));
@@ -1216,6 +1276,10 @@ static void cb_font_double_underline (GtkToggleAction *act, WorkbookControlGUI *
 	{ toggle_font_attr (wbcg, act, MSTYLE_FONT_UNDERLINE, UNDERLINE_DOUBLE, UNDERLINE_NONE); }
 static void cb_font_strikethrough (GtkToggleAction *act, WorkbookControlGUI *wbcg)
 	{ toggle_font_attr (wbcg, act, MSTYLE_FONT_STRIKETHROUGH, TRUE, FALSE); }
+static void cb_font_subscript (GtkToggleAction *act, WorkbookControlGUI *wbcg)
+	{ toggle_font_attr (wbcg, act, MSTYLE_FONT_SCRIPT, GO_FONT_SCRIPT_SUB, GO_FONT_SCRIPT_STANDARD); }
+static void cb_font_superscript (GtkToggleAction *act, WorkbookControlGUI *wbcg)
+	{ toggle_font_attr (wbcg, act, MSTYLE_FONT_SCRIPT, GO_FONT_SCRIPT_SUPER, GO_FONT_SCRIPT_STANDARD); }
 
 static void
 apply_number_format (WorkbookControlGUI *wbcg,
@@ -1226,6 +1290,11 @@ apply_number_format (WorkbookControlGUI *wbcg,
 	cmd_selection_format (WORKBOOK_CONTROL (wbcg), mstyle, NULL, descriptor);
 }
 
+static GNM_ACTION_DEF (cb_format_as_general)
+{
+	apply_number_format (wbcg,
+		go_format_builtins [GO_FORMAT_GENERAL][0], _("Format as General"));
+}
 static GNM_ACTION_DEF (cb_format_as_number)
 {
 	apply_number_format (wbcg,
@@ -1466,6 +1535,14 @@ static GtkActionEntry const permanent_actions[] = {
 	{ "EditPaste", GTK_STOCK_PASTE, NULL,
 		NULL, N_("Paste the clipboard"),
 		G_CALLBACK (cb_edit_paste) },
+#ifdef USE_HILDON
+	{ "UndoHildon", GTK_STOCK_UNDO, N_("_Undo"), 
+		NULL, N_("Undo the last action"), 
+		G_CALLBACK (cb_edit_undo) },
+	{ "RedoHildon", GTK_STOCK_REDO, N_("_Redo"),
+		NULL, N_("Redo the undone action"), 
+		G_CALLBACK (cb_edit_redo) },
+#endif		
 
 	{ "HelpDocs", GTK_STOCK_HELP, N_("_Contents"),
 		"F1", N_("Open a viewer for Gnumeric's documentation"),
@@ -1574,7 +1651,7 @@ static GtkActionEntry const actions[] = {
 		"<control>bracketleft", N_("Select all the cells are used by the current edit cell"),
 		G_CALLBACK (cb_edit_select_inputs) },
 	{ "EditMetadata", NULL, N_("Edit Metadata"),
-		"<control>bracketleft", N_("Edit the document's extra information (metadata)"),
+		NULL, N_("Edit the document's extra information (metadata)"),
 		G_CALLBACK (cb_new_doc_metadata) },
 
 /* Edit -> Fill */
@@ -1966,14 +2043,17 @@ static GtkActionEntry const actions[] = {
 		NULL, N_("Split merged ranges of cells"),
 		G_CALLBACK (cb_unmerge_cells) },
 
+	{ "FormatAsGeneral", NULL, N_("General"),
+		"<control>asciitilde", N_("Format the selection as General"),
+		G_CALLBACK (cb_format_as_general) },
 	{ "FormatAsNumber", NULL, N_("Number"),
-		"<control>asciitilde", N_("Format the selection as numbers"),
+		"<control>exclam", N_("Format the selection as numbers"),
 		G_CALLBACK (cb_format_as_number) },
 	{ "FormatAsCurrency", NULL, N_("Currency"),
 		"<control>dollar", N_("Format the selection as currency"),
 		G_CALLBACK (cb_format_as_currency) },
 	{ "FormatAsAccounting", "Gnumeric_FormatAsAccounting", N_("Accounting"),
-		"<control>exclam", N_("Format the selection as accounting"),
+		NULL, N_("Format the selection as accounting"),
 		G_CALLBACK (cb_format_as_accounting) },
 	{ "FormatAsPercentage", "Gnumeric_FormatAsPercentage", N_("Percentage"),
 		"<control>percent", N_("Format the selection as percentage"),
@@ -2109,7 +2189,7 @@ static GtkToggleActionEntry const toggle_actions[] = {
 
 static GtkToggleActionEntry const font_toggle_actions[] = {
 	{ "FontBold", GTK_STOCK_BOLD,
-		N_("_Bold"), "<control>B",	/* ALSO "<control>2" */
+		N_("_Bold"), "<control>b",	/* ALSO "<control>2" */
 		N_("Bold"), G_CALLBACK (cb_font_bold), FALSE },
 	{ "FontItalic", GTK_STOCK_ITALIC,
 		N_("_Italic"), "<control>i",	/* ALSO "<control>3" */
@@ -2117,13 +2197,18 @@ static GtkToggleActionEntry const font_toggle_actions[] = {
 	{ "FontUnderline", GTK_STOCK_UNDERLINE,
 		N_("_Underline"), "<control>u",	/* ALSO "<control>4" */
 		N_("Underline"), G_CALLBACK (cb_font_underline), FALSE },
-	{ "FontStrikeThrough", GTK_STOCK_UNDERLINE,
+	{ "FontStrikeThrough", GTK_STOCK_STRIKETHROUGH,
 		N_("_Strike Through"), "<control>5",
 		N_("Strike Through"), G_CALLBACK (cb_font_strikethrough), FALSE },
-#warning "Add double underline icon"
-	{ "FontDoubleUnderline", GTK_STOCK_UNDERLINE,
+	{ "FontDoubleUnderline", "stock_text_underlined-double",	/* from icon theme */
 		N_("_Double Underline"), "<control><shift>d",
 		N_("Double Underline"), G_CALLBACK (cb_font_double_underline), FALSE },
+	{ "FontSuperscript", "stock_superscript",	/* from icon theme */
+		N_("Su_perscript"), "<control>asciicircum",
+		N_("Superscript"), G_CALLBACK (cb_font_superscript), FALSE },
+	{ "FontSubscript", "stock_subscript",	/* from icon theme */
+		N_("Subscrip_t"), "<control>underscore",
+		N_("Subscript"), G_CALLBACK (cb_font_subscript), FALSE }
 };
 
 void wbcg_register_actions (WorkbookControlGUI *wbcg,
