@@ -54,6 +54,7 @@
 #include "gnm-so-filled.h"
 #include "sheet-object-graph.h"
 #include "application.h"
+#include "xml-io.h"
 
 #include <goffice/app/io-context.h>
 #include <goffice/app/go-plugin.h>
@@ -76,8 +77,8 @@ GNM_PLUGIN_MODULE_HEADER;
 
 /*****************************************************************************/
 
-static gboolean
-xml_sax_attr_double (xmlChar const * const *attrs, char const *name, double * res)
+gboolean
+gnm_xml_attr_double (xmlChar const * const *attrs, char const *name, double * res)
 {
 	char *end;
 	double tmp;
@@ -298,31 +299,37 @@ typedef struct {
 	SheetObject *so;
 } XMLSaxParseState;
 
+SheetObject *
+gnm_xml_in_cur_obj (GsfXMLIn const *xin)
+{
+	XMLSaxParseState *state = (XMLSaxParseState *)xin->user_state;
+	return state->so;
+}
+
 /****************************************************************************/
 
 static void
-unknown_attr (XMLSaxParseState *state,
-	      xmlChar const * const *attrs)
+unknown_attr (GsfXMLIn *gsf_state, xmlChar const * const *attrs)
 {
-	g_return_if_fail (state != NULL);
-	g_return_if_fail (attrs != NULL);
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	if (state->version == GNM_XML_LATEST)
 		gnm_io_warning (state->context,
 			_("Unexpected attribute %s::%s == '%s'."),
-			(NULL != state->base.node &&
-			 NULL != state->base.node->name) ?
-			state->base.node->name : "<unknow name>",
+			(NULL != gsf_state->node &&
+			 NULL != gsf_state->node->name) ?
+			gsf_state->node->name : "<unknow name>",
 			attrs[0], attrs[1]);
 }
 
 static void
 xml_sax_wb (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (strcmp (attrs[0], "xmlns:gmr") == 0) {
+		if (strcmp (attrs[0], "xmlns:gmr") == 0 ||
+		    strcmp (attrs[0], "xmlns:gnm") == 0) {
 			static struct {
 				char const * const id;
 				GnumericXMLVersion const version;
@@ -354,23 +361,25 @@ xml_sax_wb (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		} else if (!strcmp (attrs[0], "xmlns:xsi")) {
 		} else if (!strcmp (attrs[0], "xsi:schemaLocation")) {
 		} else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 }
 
 static void
 xml_sax_wb_sheetname (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
+	char const *name = gsf_state->content->str;
+	
+	g_return_if_fail (name != NULL);
 
-	char const *content = state->base.content->str;
-	Sheet *sheet = sheet_new (state->wb, content);
-	workbook_sheet_attach (state->wb, sheet);
+	if (NULL == workbook_sheet_by_name (state->wb, name))
+		workbook_sheet_attach (state->wb, sheet_new (state->wb, name));
 }
 
 static void
 xml_sax_wb_view (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int sheet_index;
 	int width = -1, height = -1;
@@ -382,7 +391,7 @@ xml_sax_wb_view (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (xml_sax_attr_int (attrs, "Width", &width)) ;
 		else if (xml_sax_attr_int (attrs, "Height", &height)) ;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 
 	if (width > 0 && height > 0)
 		wb_view_preferred_size (state->wb_view, width, height);
@@ -390,7 +399,7 @@ xml_sax_wb_view (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_calculation (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	gboolean b;
 	int 	 i;
 	double	 d;
@@ -402,16 +411,16 @@ xml_sax_calculation (GsfXMLIn *gsf_state, xmlChar const **attrs)
 			workbook_iteration_enabled (state->wb, b);
 		else if (xml_sax_attr_int  (attrs, "MaxIterations", &i))
 			workbook_iteration_max_number (state->wb, i);
-		else if (xml_sax_attr_double (attrs, "IterationTolerance", &d))
+		else if (gnm_xml_attr_double (attrs, "IterationTolerance", &d))
 			workbook_iteration_tolerance (state->wb, d);
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 }
 
 static void
 xml_sax_finish_parse_wb_attr (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->attribute.name != NULL);
 	g_return_if_fail (state->attribute.value != NULL);
@@ -426,10 +435,10 @@ xml_sax_finish_parse_wb_attr (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blo
 static void
 xml_sax_attr_elem (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
-	char const *content = state->base.content->str;
-	int const len = state->base.content->len;
+	char const *content = gsf_state->content->str;
+	int const len = gsf_state->content->len;
 
 	switch (gsf_state->node->user_data.v_int) {
 	case 0 :
@@ -450,7 +459,7 @@ xml_sax_attr_elem (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_sheet_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	gboolean tmp;
 	gint tmpi;
@@ -488,13 +497,13 @@ xml_sax_sheet_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (xml_sax_attr_color (attrs, "TabColor", &color))
 			state->tab_color = color;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 }
 
 static void
 xml_sax_sheet_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->sheet != NULL);
 
@@ -507,9 +516,9 @@ xml_sax_sheet_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_sheet_name (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
-	char const * content = state->base.content->str;
+	char const * content = gsf_state->content->str;
 	g_return_if_fail (state->sheet == NULL);
 
 	/* * FIXME: Pull this out at some point, so we don't
@@ -551,9 +560,9 @@ xml_sax_sheet_name (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_sheet_zoom (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
-	char const * content = state->base.content->str;
+	char const * content = gsf_state->content->str;
 	double zoom;
 
 	g_return_if_fail (state->sheet != NULL);
@@ -563,36 +572,36 @@ xml_sax_sheet_zoom (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 }
 
 static double
-xml_sax_print_margins_get_double (XMLSaxParseState *state, xmlChar const **attrs)
+xml_sax_print_margins_get_double (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
 	double points;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
-		if (xml_sax_attr_double (attrs, "Points", &points))
+		if (gnm_xml_attr_double (attrs, "Points", &points))
 			return points;
 		else if (strcmp (attrs[0], "PrefUnit"))
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 	return 0.0;
 }
 
 static void
-xml_sax_print_margins_unit (XMLSaxParseState *state, xmlChar const **attrs, PrintUnit *pu)
+xml_sax_print_margins_unit (GsfXMLIn *gsf_state, xmlChar const **attrs, PrintUnit *pu)
 {
 	double points;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
-		if (xml_sax_attr_double (attrs, "Points", &points))
+		if (gnm_xml_attr_double (attrs, "Points", &points))
 			pu->points = points;
 		else if (!strcmp (attrs[0], "PrefUnit")) {
 			pu->desired_display = unit_name_to_unit (attrs[1]);
 		} else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 }
 
 static void
 xml_sax_print_margins (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	PrintInformation *pi;
 
@@ -601,23 +610,23 @@ xml_sax_print_margins (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	pi = state->sheet->print_info;
 	switch (gsf_state->node->user_data.v_int) {
-	case 0: xml_sax_print_margins_unit (state, attrs,
+	case 0: xml_sax_print_margins_unit (gsf_state, attrs,
 			&pi->margin.top);
 		break;
-	case 1: xml_sax_print_margins_unit (state, attrs,
+	case 1: xml_sax_print_margins_unit (gsf_state, attrs,
 			&pi->margin.bottom);
 		break;
 	case 2: print_info_set_margin_left (pi,
-			xml_sax_print_margins_get_double (state, attrs));
+			xml_sax_print_margins_get_double (gsf_state, attrs));
 		break;
 	case 3: print_info_set_margin_right (pi,
-			xml_sax_print_margins_get_double (state, attrs));
+			xml_sax_print_margins_get_double (gsf_state, attrs));
 		break;
 	case 4: print_info_set_margin_header (pi,
-			xml_sax_print_margins_get_double (state, attrs));
+			xml_sax_print_margins_get_double (gsf_state, attrs));
 		break;
 	case 5: print_info_set_margin_footer (pi,
-			xml_sax_print_margins_get_double (state, attrs));
+			xml_sax_print_margins_get_double (gsf_state, attrs));
 		break;
 	default:
 		return;
@@ -630,7 +639,7 @@ xml_sax_print_margins (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_print_scale (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	PrintInformation *pi;
 	double percentage;
@@ -644,7 +653,7 @@ xml_sax_print_scale (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		if (!strcmp (attrs[0], "type"))
 			pi->scaling.type = strcmp (attrs[1], "percentage")
 				? PRINT_SCALE_FIT_PAGES : PRINT_SCALE_PERCENTAGE;
-		else if (xml_sax_attr_double (attrs, "percentage", &percentage))
+		else if (gnm_xml_attr_double (attrs, "percentage", &percentage))
 			pi->scaling.percentage.x = pi->scaling.percentage.y = percentage;
 		else if (xml_sax_attr_int (attrs, "cols", &cols))
 			pi->scaling.dim.cols = cols;
@@ -656,7 +665,7 @@ xml_sax_print_scale (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_selection_range (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmRange r;
 	if (xml_sax_attr_range (attrs, &r))
@@ -670,7 +679,7 @@ xml_sax_selection_range (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_selection (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int col = -1, row = -1;
 
@@ -680,7 +689,7 @@ xml_sax_selection (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		if (xml_sax_attr_int (attrs, "CursorCol", &col)) ;
 		else if (xml_sax_attr_int (attrs, "CursorRow", &row)) ;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 
 	g_return_if_fail (col >= 0);
 	g_return_if_fail (row >= 0);
@@ -693,7 +702,7 @@ xml_sax_selection (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_selection_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmCellPos const pos = state->cell;
 	state->cell.col = state->cell.row = -1;
@@ -703,7 +712,7 @@ xml_sax_selection_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_sheet_layout (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmCellPos tmp;
 
@@ -713,13 +722,13 @@ xml_sax_sheet_layout (GsfXMLIn *gsf_state, xmlChar const **attrs)
 				sheet_get_view (state->sheet, state->wb_view),
 				tmp.col, tmp.row);
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 }
 
 static void
 xml_sax_sheet_freezepanes (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmCellPos frozen_tl, unfrozen_tl;
 	int flags = 0;
@@ -730,7 +739,7 @@ xml_sax_sheet_freezepanes (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (xml_sax_attr_cellpos (attrs, "UnfrozenTopLeft", &unfrozen_tl))
 			flags |= 2;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 
 	if (flags == 3)
 		sv_freeze_panes (sheet_get_view (state->sheet, state->wb_view),
@@ -740,7 +749,7 @@ xml_sax_sheet_freezepanes (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_cols_rows (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	double def_size;
 	gboolean const is_col = gsf_state->node->user_data.v_bool;
@@ -748,7 +757,7 @@ xml_sax_cols_rows (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	g_return_if_fail (state->sheet != NULL);
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (xml_sax_attr_double (attrs, "DefaultSizePts", &def_size)) {
+		if (gnm_xml_attr_double (attrs, "DefaultSizePts", &def_size)) {
 			if (is_col)
 				sheet_col_set_default_size_pts (state->sheet, def_size);
 			else
@@ -759,7 +768,7 @@ xml_sax_cols_rows (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	ColRowInfo *cri = NULL;
 	double size = -1.;
@@ -779,7 +788,7 @@ xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		} else {
 			g_return_if_fail (cri != NULL);
 
-			if (xml_sax_attr_double (attrs, "Unit", &size)) ;
+			if (gnm_xml_attr_double (attrs, "Unit", &size)) ;
 			else if (xml_sax_attr_int (attrs, "Count", &count)) ;
 			else if (xml_sax_attr_int (attrs, "MarginA", &dummy))
 				cri->margin_a = dummy;
@@ -794,7 +803,7 @@ xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 			else if (xml_sax_attr_int (attrs, "OutlineLevel", &dummy))
 				cri->outline_level = dummy;
 			else
-				unknown_attr (state, attrs);
+				unknown_attr (gsf_state, attrs);
 		}
 	}
 
@@ -818,7 +827,7 @@ xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_style_region_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->style_range_init == FALSE);
 	g_return_if_fail (state->style == NULL);
@@ -834,7 +843,7 @@ xml_sax_style_region_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_style_region_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->style_range_init);
 	g_return_if_fail (state->style != NULL);
@@ -849,7 +858,7 @@ xml_sax_style_region_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_styleregion_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int val;
 	GnmColor *colour;
@@ -898,14 +907,14 @@ xml_sax_styleregion_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (xml_sax_attr_int (attrs, "Orient", &val))
 			; /* ignore old useless attribute */
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 }
 
 static void
 xml_sax_styleregion_font (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	double size_pts = 10.;
 	int val;
@@ -913,7 +922,7 @@ xml_sax_styleregion_font (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	g_return_if_fail (state->style != NULL);
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
-		if (xml_sax_attr_double (attrs, "Unit", &size_pts))
+		if (gnm_xml_attr_double (attrs, "Unit", &size_pts))
 			gnm_style_set_font_size (state->style, size_pts);
 		else if (xml_sax_attr_int (attrs, "Bold", &val))
 			gnm_style_set_font_bold (state->style, val);
@@ -923,8 +932,15 @@ xml_sax_styleregion_font (GsfXMLIn *gsf_state, xmlChar const **attrs)
 			gnm_style_set_font_uline (state->style, (GnmUnderline)val);
 		else if (xml_sax_attr_int (attrs, "StrikeThrough", &val))
 			gnm_style_set_font_strike (state->style, val ? TRUE : FALSE);
-		else
-			unknown_attr (state, attrs);
+		else if (xml_sax_attr_int (attrs, "Script", &val)) {
+			if (val == 0)
+				gnm_style_set_font_script (state->style, GO_FONT_SCRIPT_STANDARD);
+			else if (val < 0)
+				gnm_style_set_font_script (state->style, GO_FONT_SCRIPT_SUB);
+			else
+				gnm_style_set_font_script (state->style, GO_FONT_SCRIPT_SUPER);
+		} else
+			unknown_attr (gsf_state, attrs);
 	}
 }
 
@@ -977,10 +993,10 @@ style_font_read_from_x11 (GnmStyle *mstyle, const char *fontname)
 static void
 xml_sax_styleregion_font_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
-	if (state->base.content->len > 0) {
-		char const * content = state->base.content->str;
+	if (gsf_state->content->len > 0) {
+		char const * content = gsf_state->content->str;
 		if (*content == '-')
 			style_font_read_from_x11 (state->style, content);
 		else
@@ -991,7 +1007,7 @@ xml_sax_styleregion_font_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blo
 static void
 xml_sax_validation (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int dummy;
 	gboolean b_dummy;
@@ -1023,14 +1039,14 @@ xml_sax_validation (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		} else if (xml_sax_attr_bool (attrs, "UseDropdown", &b_dummy)) {
 			state->validation.use_dropdown = b_dummy;
 		} else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 }
 
 static void
 xml_sax_validation_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->style != NULL);
 
@@ -1055,7 +1071,7 @@ xml_sax_validation_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_validation_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int const i = gsf_state->node->user_data.v_int;
 	GnmExpr const *expr;
@@ -1063,7 +1079,7 @@ xml_sax_validation_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob
 
 	g_return_if_fail (state->validation.expr [i] == NULL);
 
-	expr = gnm_expr_parse_str_simple (state->base.content->str,
+	expr = gnm_expr_parse_str_simple (gsf_state->content->str,
 		parse_pos_init_sheet (&pos, state->sheet));
 
 	g_return_if_fail (expr != NULL);
@@ -1074,7 +1090,7 @@ xml_sax_validation_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob
 static void
 xml_sax_condition (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int dummy;
 
@@ -1086,13 +1102,13 @@ xml_sax_condition (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		if (xml_sax_attr_int (attrs, "Operator", &dummy))
 			state->cond.op = dummy;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 }
 
 static void
 xml_sax_condition_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	GnmStyleConditions *sc;
 
 	g_return_if_fail (state->style != NULL);
@@ -1109,7 +1125,7 @@ xml_sax_condition_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_condition_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int const i = gsf_state->node->user_data.v_int;
 	GnmExpr const *expr;
@@ -1117,7 +1133,7 @@ xml_sax_condition_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 	g_return_if_fail (state->cond.expr [i] == NULL);
 
-	expr = gnm_expr_parse_str_simple (state->base.content->str,
+	expr = gnm_expr_parse_str_simple (gsf_state->content->str,
 		parse_pos_init_sheet (&pos, state->sheet));
 
 	g_return_if_fail (expr != NULL);
@@ -1128,7 +1144,7 @@ xml_sax_condition_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_hlink (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	char *type = NULL;
 	char *target = NULL;
 	char *tip = NULL;
@@ -1143,7 +1159,7 @@ xml_sax_hlink (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (!strcmp (attrs[0], "tip"))
 			target = g_strdup ((gchar *)attrs[1]);
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 
 	if (NULL != type && NULL != target) {
@@ -1162,7 +1178,7 @@ xml_sax_hlink (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_input_msg (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	char *title = NULL;
 	char *msg = NULL;
 
@@ -1174,7 +1190,7 @@ xml_sax_input_msg (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (!strcmp (attrs[0], "Message"))
 			msg = g_strdup ((gchar *)attrs[1]);
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 
 	gnm_style_set_input_msg (state->style,
@@ -1186,7 +1202,7 @@ xml_sax_input_msg (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int pattern = -1;
 	GnmColor *colour = NULL;
@@ -1198,7 +1214,7 @@ xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		if (xml_sax_attr_color (attrs, "Color", &colour)) ;
 		else if (xml_sax_attr_int (attrs, "Style", &pattern)) ;
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 
 	if (pattern >= STYLE_BORDER_NONE) {
@@ -1213,7 +1229,7 @@ xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 static void
 xml_sax_cell (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int row = -1, col = -1;
 	int rows = -1, cols = -1;
@@ -1238,7 +1254,7 @@ xml_sax_cell (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (!strcmp (attrs[0], "ValueFormat"))
 			value_fmt = go_format_new_from_XL ((char *)attrs[1], FALSE);
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs);
 	}
 
 	g_return_if_fail (col >= 0);
@@ -1334,7 +1350,7 @@ xml_not_used_old_array_spec (GnmCell *cell, char const *content)
 static void
 xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	gboolean is_new_cell, is_post_52_array = FALSE;
 	GnmCell *cell;
@@ -1371,8 +1387,8 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 	is_post_52_array = (array_cols > 0) && (array_rows > 0);
 
-	if (state->base.content->len > 0) {
-		char const * content = state->base.content->str;
+	if (gsf_state->content->len > 0) {
+		char const * content = gsf_state->content->str;
 
 		if (is_post_52_array) {
 			g_return_if_fail (content[0] == '=');
@@ -1429,12 +1445,12 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_merge (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmRange r;
-	g_return_if_fail (state->base.content->len > 0);
+	g_return_if_fail (gsf_state->content->len > 0);
 
-	if (parse_range (state->base.content->str, &r))
+	if (parse_range (gsf_state->content->str, &r))
 		sheet_merge_add (state->sheet, &r, FALSE,
 			GO_CMD_CONTEXT (state->context));
 }
@@ -1442,10 +1458,11 @@ xml_sax_merge (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_object_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	char const *type_name = gsf_state->node->name;
-	int tmp_int;
+	int tmp_int, i;
 	SheetObject *so;
+	SheetObjectClass *klass;
 
 	g_return_if_fail (state->so == NULL);
 
@@ -1494,32 +1511,38 @@ xml_sax_object_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	state->so = so;
 
 	so->anchor.direction = SO_DIR_UNKNOWN;
-	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
-		if (!strcmp (attrs[0], "ObjectBound")) {
+	for (i = 0; attrs != NULL && attrs[i] && attrs[i+1] ; i += 2) {
+		if (!strcmp (attrs[i], "ObjectBound")) {
 			GnmRange r;
-			if (parse_range (attrs[1], &r))
+			if (parse_range (attrs[i+1], &r))
 				so->anchor.cell_bound = r;
-		} else if (!strcmp (attrs[0], "ObjectOffset")) {
-			sscanf (attrs[1], "%g %g %g %g",
+		} else if (!strcmp (attrs[i], "ObjectOffset")) {
+			sscanf (attrs[i+1], "%g %g %g %g",
 				so->anchor.offset +0, so->anchor.offset +1,
 				so->anchor.offset +2, so->anchor.offset +3);
-		} else if (!strcmp (attrs[0], "ObjectanchorType")) {
-			int i[4], count;
-			sscanf (attrs[1], "%d %d %d %d", i+0, i+1, i+2, i+3);
+		} else if (!strcmp (attrs[i], "ObjectAnchorType")) {
+			int n[4], count;
+			sscanf (attrs[i+1], "%d %d %d %d", n+0, n+1, n+2, n+3);
 
 			for (count = 4; count-- > 0 ; )
-				so->anchor.type[count] = i[count];
-		} else if (xml_sax_attr_int (attrs, "Direction", &tmp_int))
+				so->anchor.type[count] = n[count];
+		} else if (xml_sax_attr_int (attrs+i, "Direction", &tmp_int))
 			so->anchor.direction = tmp_int;
+#if 0 /* There may be extra attributes that are handled by the objects */
 		else
-			unknown_attr (state, attrs);
+			unknown_attr (gsf_state, attrs+i);
+#endif
 	}
+
+	klass = SHEET_OBJECT_CLASS (G_OBJECT_GET_CLASS (so));
+	if (klass->prep_sax_parser)
+		(klass->prep_sax_parser) (so, gsf_state, attrs);
 }
 
 static void
 xml_sax_object_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	sheet_object_set_sheet (state->so, state->sheet);
 	g_object_unref (state->so);
 	state->so = NULL;
@@ -1528,7 +1551,7 @@ xml_sax_object_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_named_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	GnmParseError  perr;
 	GnmParsePos    pos;
@@ -1579,10 +1602,10 @@ xml_sax_named_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_named_expr_prop (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
-	char const * content = state->base.content->str;
-	int const len = state->base.content->len;
+	char const * content = gsf_state->content->str;
+	int const len = gsf_state->content->len;
 
 	switch (gsf_state->node->user_data.v_int) {
 	case 0:
@@ -1605,34 +1628,31 @@ xml_sax_named_expr_prop (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 static void
 xml_sax_orientation (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 	PrintInformation *pi;
 
 	g_return_if_fail (state->sheet != NULL);
 	g_return_if_fail (state->sheet->print_info != NULL);
 
 	pi = state->sheet->print_info;
-	pi->portrait_orientation = (strcmp (state->base.content->str, "portrait") == 0);
+	pi->portrait_orientation = (strcmp (gsf_state->content->str, "portrait") == 0);
 }
 
 static void
 xml_sax_paper (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 {
-	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state;
-	char const *content = state->base.content->str;
+	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	g_return_if_fail (state->sheet != NULL);
 	g_return_if_fail (state->sheet->print_info != NULL);
 
-	print_info_set_paper (state->sheet->print_info, content);
+	print_info_set_paper (state->sheet->print_info, gsf_state->content->str);
 }
 
-
 /****************************************************************************/
 
 #define GNM	0
 
-G_GNUC_UNUSED
 static GsfXMLInNS content_ns[] = {
 	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v10.dtd"),
 	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v9.dtd"),
@@ -1647,7 +1667,6 @@ static GsfXMLInNS content_ns[] = {
 	{ NULL }
 };
 
-G_GNUC_UNUSED
 static GsfXMLInNode gnumeric_1_0_dtd[] = {
 GSF_XML_IN_NODE_FULL (START, START, -1, NULL, FALSE, FALSE, TRUE, NULL, NULL, 0),
 GSF_XML_IN_NODE_FULL (START, WB, GNM, "Workbook", FALSE, TRUE, FALSE, &xml_sax_wb, NULL, 0),
@@ -1672,6 +1691,10 @@ GSF_XML_IN_NODE_FULL (START, WB, GNM, "Workbook", FALSE, TRUE, FALSE, &xml_sax_w
       GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_NAME,	   GNM, "name",	    TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 0),
       GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_VALUE,	   GNM, "value",    TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 1),
       GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_POSITION, GNM, "position", TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 2),
+      /* sometimes not namespaced */
+      GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_NAME_NS,     -1, "name",     TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 0),
+      GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_VALUE_NS,    -1, "value",    TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 1),
+      GSF_XML_IN_NODE_FULL (WB_NAMED_EXPR, WB_NAMED_EXPR_POSITION_NS, -1, "position", TRUE, FALSE, FALSE, NULL, &xml_sax_named_expr_prop, 2),
 
   GSF_XML_IN_NODE (WB, WB_SHEETS, GNM, "Sheets", FALSE, NULL, NULL),
     GSF_XML_IN_NODE (WB_SHEETS, SHEET, GNM, "Sheet", FALSE, &xml_sax_sheet_start, &xml_sax_sheet_end),
@@ -1766,7 +1789,8 @@ GSF_XML_IN_NODE_FULL (START, WB, GNM, "Workbook", FALSE, TRUE, FALSE, &xml_sax_w
 	GSF_XML_IN_NODE (SHEET_LAYOUT, SHEET_FREEZEPANES, GNM, "FreezePanes", FALSE, &xml_sax_sheet_freezepanes, NULL),
 
       GSF_XML_IN_NODE (SHEET, SHEET_SOLVER, GNM, "Solver", FALSE, NULL, NULL),
-#warning TODO : add Solver import
+      GSF_XML_IN_NODE (SHEET, SHEET_SCENARIOS, GNM, "Scenarios", FALSE, NULL, NULL),
+        GSF_XML_IN_NODE (SHEET_SCENARIOS, SHEET_SCENARIO, GNM, "Scenario", FALSE, NULL, NULL),
 
       GSF_XML_IN_NODE (SHEET, SHEET_OBJECTS, GNM, "Objects", FALSE, NULL, NULL),
         /* Old crufty IO */
@@ -1782,13 +1806,14 @@ GSF_XML_IN_NODE_FULL (START, WB, GNM, "Workbook", FALSE, TRUE, FALSE, &xml_sax_w
 	GSF_XML_IN_NODE (SHEET_OBJECTS, OBJECT_OLD_LINE, GNM, "SheetObjectGraphic", FALSE, &xml_sax_object_start, &xml_sax_object_end),
 	GSF_XML_IN_NODE (SHEET_OBJECTS, OBJECT_OLD_FILLED, GNM, "SheetObjectFilled", FALSE, &xml_sax_object_start, &xml_sax_object_end),
 	GSF_XML_IN_NODE (SHEET_OBJECTS, OBJECT_OLD_TEXT, GNM, "SheetObjectText", FALSE,	&xml_sax_object_start, &xml_sax_object_end),
+	GSF_XML_IN_NODE (SHEET_OBJECTS, OBJECT_GRAPH, GNM, "SheetObjectGraph", FALSE,	&xml_sax_object_start, &xml_sax_object_end),
+	GSF_XML_IN_NODE (SHEET_OBJECTS, OBJECT_IMAGE, GNM, "SheetObjectImage", FALSE,	&xml_sax_object_start, &xml_sax_object_end),
 
   GSF_XML_IN_NODE (WB, WB_GEOMETRY, GNM, "Geometry", FALSE, &xml_sax_wb_view, NULL),
   GSF_XML_IN_NODE (WB, WB_VIEW, GNM, "UIData", FALSE, &xml_sax_wb_view, NULL),
   GSF_XML_IN_NODE (WB, WB_CALC, GNM, "Calculation", FALSE, &xml_sax_calculation, NULL),
   { NULL }
 };
-static GsfXMLInDoc *gnm_sax_in_doc;
 
 #if 0
 I do not like this interface
@@ -1893,27 +1918,19 @@ gnm_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 		   gpointer wb_view, GsfInput *input)
 {
 	XMLSaxParseState state;
+	GsfXMLInDoc     *doc;
 	char *old_num_locale, *old_monetary_locale;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wb_view));
 	g_return_if_fail (GSF_IS_INPUT (input));
 
-	if (NULL == gnm_sax_in_doc) {
-		gnm_sax_in_doc = gsf_xml_in_doc_new (gnumeric_1_0_dtd, content_ns);
+	doc = gsf_xml_in_doc_new (gnumeric_1_0_dtd, content_ns);
+	if (doc == NULL)
+		return;
 #if 0
-		gsf_xml_in_doc_set_unknown_handler (gnm_sax_in_doc, &xml_sax_unknown);
+	gsf_xml_in_doc_set_unknown_handler (gnm_sax_in_doc, &xml_sax_unknown);
 #endif
-#warning I doubt the life cycle issues of nodes from plugins are worth the gain from caching this
-		g_object_set_data_full (gnm_app_get_app (),
-			"GnmXMLSaxDoc", gnm_sax_in_doc,
-			(GDestroyNotify) gsf_xml_in_doc_free);
-	}
-
-	g_return_if_fail (gnm_sax_in_doc != NULL);
-
 	/* init */
-	state.base.doc = gnm_sax_in_doc;
-
 	state.context = io_context;
 	state.wb_view = wb_view;
 	state.wb = wb_view_workbook (wb_view);
@@ -1945,7 +1962,7 @@ gnm_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 	go_setlocale (LC_MONETARY, "C");
 	go_set_untranslated_bools ();
 
-	if (!gsf_xml_in_parse (&state.base, input))
+	if (!gsf_xml_in_doc_parse (doc, input, &state))
 		gnumeric_io_error_string (io_context, _("XML document not well formed!"));
 	else
 		workbook_queue_all_recalc (state.wb);
@@ -1960,4 +1977,6 @@ gnm_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 
 	/* cleanup */
 	g_hash_table_destroy (state.expr_map);
+
+	gsf_xml_in_doc_free (doc);
 }
